/*
 * Button debounce and logic functions
 *
 * Button logic detects two conditions that can be read after each evaluation
 *    1. Short Button pressed - button has been released before the long press timeout
 *    2. Long Button pressed - button has been held past long press timeout (signaled at timeout)
 *
 */

//
// Digital IO Pin - Button sense is active low
//
#define nBUTTON      A3

//
// Long press threshold (seconds)
//
#define BUTTON_LONG_PRESS_TO  2

//
// Variables
//
uint16_t _buttonLongPressTimer;
boolean _prevButton;
boolean _buttonDown;
boolean _buttonShortPress;
boolean _buttonLongPress;


//
// API routines
//

void InitButton() {
  // Configure IO
  pinMode(nBUTTON, INPUT);
  
  // Set initial state (button is pressed when we power-up)
  _buttonLongPressTimer = 0;
  _prevButton = true;
  _buttonDown = true;
  _buttonShortPress = false;
  _buttonLongPress = false;
}


void EvalButton() {
  boolean curButton;
  boolean buttonPressed;
  boolean buttonReleased;
  
  // Status set as necessary
  _buttonShortPress = false;
  _buttonLongPress = false;
  
  // Evaluate each timer tick (debounce interval)
  if (timerTick) {
    // Get the current button value
    curButton = digitalRead(nBUTTON) == LOW;
    
    // Evalute button state
    buttonPressed = curButton && _prevButton && !_buttonDown;
    buttonReleased = _buttonDown && !curButton && !_prevButton;
    if (curButton && _prevButton) {
      _buttonDown = true;
    } else if (!curButton && !_prevButton) {
      _buttonDown = false;
    }
    _prevButton = curButton;
    
    // Evalute button status
    if (buttonPressed) {
      // Initialize timer
      _buttonLongPressTimer = BUTTON_LONG_PRESS_TO * (TIMER_INIT / 1000);
    } else {
      if (buttonReleased && (_buttonLongPressTimer != 0)) {
        // Short press detected
        _buttonShortPress = true;
      } else if (_buttonDown) {
        // Look for long press detection
        if (_buttonLongPressTimer != 0) {
          if (--_buttonLongPressTimer == 0) {
            _buttonLongPress = true;
          }
        }
      }
    }
  }
}

boolean ButtonShortPress() {
  return _buttonShortPress;
}

boolean ButtonLongPress() {
  return _buttonLongPress;
}
